###
### $Id: Pub.tcl,v 1.12 2002/11/09 20:31:47 wcc Exp $
###

##################################
### Pub.tcl                    ###
### Version 1.8                ###
### By Wcc                     ###
### wcc@techmonkeys.org        ###
### http://www.dawgtcl.com:81/ ###
### EFnet #|DAWG|Tcl           ###
##################################

############################################################################
### Copyright  2000 - 2002 |DAWG| Scripting Group. All rights reserved. ###
############################################################################

###############################################################################
## This script provides 53 in-channel commands for interacting with the bot. ##
## Auth and SubOp are supported.                                             ##
###############################################################################

##############
## COMMANDS ##
###################################################################
## PUB ## <command prefix> <command>                             ##
######### Use <command prefix> help for a full list of commands. ##
###################################################################
## MSG ## /msg <bot> auth <password>                             ##
######### Authorizes you to use pub commands.                    ##
######### ------------------------------------------------------ ##
######### /msg <bot> deauth                                      ##
######### De-authorizes you from using pub commands.             ##
###################################################################
## DCC ## .pubhelp                                               ##
######### Shows script help.                                     ##
######### ------------------------------------------------------ ##
######### .chanset <channel> +/-subop                            ##
######### Enables or disables subop for the specified channel.   ##
######### ------------------------------------------------------ ##
######### .chanset <channel> +/-pubauth                          ##
######### Enables or disables auth for the specified channel.    ##
###################################################################

##########################################################
## Just load the script, set the variables, and rehash. ##
##########################################################

##############################################################################
# Set the command prefixes (seperated by a space) for all pub commands here. #
# You can use "%botnick" to use the bot's nick. "%botnetnick" is the bot's   #
# botnetnick.                                                                #
##############################################################################

set pub_setting(prefix) "%botnetnick dawg %botnick"

###########################
# Enable the die command? #
###########################

set pub_setting(enable-die) 0

##############################################################################
# Allow subops to use the bot to preform some op commands on +subop channels #
# without ops?                                                               #
##############################################################################

set pub_setting(subop) 1

#######################################
# Set the flag to give sub-opd users. #
#######################################

set pub_setting(subop_flag) "S"

###################################
# Force auth on for all channels? #
###################################

set pub_setting(allauth) 0

###############################################################################
# Advanced: You can override the reply of <command prefix> uptime by setting  #
# it here. Default setting is recommended. Available variables are: %version, #
# %ostype, %osversion, %osbyteorder, %user, %machine, %uptime, and %platform. #
# %bold inserts a bold control code.                                          #
###############################################################################

set pub_setting(version) "Eggdrop %version on %ostype %osversion (Uptime: %uptime)."

################################################################################
# Advanced: Set the version reply to send non-master users here.  Available    #
# variables are: %version, %ostype, %osversion, %osbyteorder, %user, %machine, #
# %uptime, and %platform. %bold inserts a bold control code.                   #
################################################################################

set pub_setting(hversion) "Eggdrop %version."

################################################################
# Send a notice when a nick does not have access to a command? #
################################################################

set pub_setting(noticeacc) 1

#######################################################
# Enable use of bold in some messages to the channel? #
#######################################################

set pub_setting(bold) 1

###################################
# Enable use of bold in DCC chat? #
###################################

set pub_setting(dbold) 1

#######################################
# Prefix "PUB:" in DCC chat messages? #
#######################################

set pub_setting(PUB:) 1

##################################################################
# Set the encryption key here. (Used for the "randpass" command) #
##################################################################

set pub_setting(key) "bd4GGfh47H36dh43"

######################################################################
# Set the 2nd encryption key here. (Used for the "randpass" command) #
######################################################################

set pub_setting(rand) "99544684584"

####################
# Code begins here #
####################

if {$numversion < 1061000 || $numversion > 1070000} { putlog "\002PUB:\002 \002WARNING:\002 This script is intended to run on eggdrop 1.6.10 or later 1.6.x versions." }
if {[info tclversion] < 8.2} { putlog "\002PUB:\002 \002WARNING:\002 This script is intended to run on Tcl Version 8.2 or later." }

foreach {a b c} [list pubm * pub_parsecmd dcc pubhelp pub_help_dcc msg auth pub_auth msg deauth pub_deauth part * pub_deauth_pq sign * pub_deauth_pq splt * pub_deauth_pq] { bind $a - $b $c }
foreach a [list subop pubauth] { setudef flag $a }

set pub_setting(commandlist) [list access gethand invite die chattr randpass -chan +chan raw restart disable enable rehash jump hosts uptime chanlist \
  +host -host topic banunban global cycle subop aop master owner autovoice wipe act say akick msg save mode voice devoice \
  op deop deopme opme voiceme kick help version devoiceme unixtime chatme halfop dehalfop halfopme dehalfopme shelluptime]

proc pub_dopre {} {
  if {!$::pub_setting(PUB:)} { return "" }
  if {!$::pub_setting(dbold)} { return "PUB: " }
  return "\002PUB:\002 "
}

proc pub_bold {text} {
  return [expr {($::pub_setting(bold)) ? "\002$text\002" : "$text"}]
}

proc pub_putmsg {chan text} {
  putserv "PRIVMSG $chan :$text"
}

proc pub_ncm {pattern string} {
  return [pub_cm [string tolower $pattern] [string tolower $string]]
}

proc pub_cm {pattern string} {
  return [string match [string tolower $pattern] [string tolower $string]]
}

proc pub_ncc {pattern string} {
  return [pub_cc [string tolower $pattern] [string tolower $string]]
}

proc pub_cc {pattern string} {
  return [expr {([string compare $pattern $string] == 0) ? 1 : 0}]
}

proc pub_sindex {1 2} {
  return [join [lrange $1 $2 $2]]
}

proc pub_putdcc {idx text} {
  putdcc $idx "[pub_dopre]$text"
}

proc pub_access_string {flags} {
  switch -- $flags {
    "n" { return "Global Owner" }
    "m" { return "Global Master" }
    "o" { return "Global Op" }
    "l" { return "Global Halfop" }
    "p" { return "Global Partyline" }
    "n|n" { return "Channel Owner" }
    "m|m" { return "Channel Master" }
    "o|o" { return "Channel Op" }
    "l|l" { return "Channel Halfop" }
  }
}

proc pub_verify_access {hand nick flags {chan "0"}} {
  set check 1
  if {![regexp {^(p|ood)$} $flags]} {
    if {[pub_cc "0" $chan] && ![matchattr $hand $flags]} {
      if {$::pub_setting(noticeacc)} { putserv "NOTICE $nick :This command requires [pub_access_string $flags] access." }
      set check 0
    }
    if {![pub_cc "0" $chan] && ![matchattr $hand $flags $chan]} {
      if {$::pub_setting(noticeacc)} { putserv "NOTICE $nick :This command requires [pub_access_string $flags] access to $chan." }
      set check 0
    }
    return $check
  }
  if {[pub_cc "p" $flags] && (![matchattr $hand p] && (![matchattr $hand o] || ${::require-p}))} { 
    if {$::pub_setting(noticeacc)} { putserv "NOTICE $nick :This command requires [pub_access_string "p"] access to $chan." }
    set check 0
  }
  if {[pub_cc "ood" $flags] && ([matchattr $hand d|d $chan] || (![matchattr $hand o|o $chan] && ![isop $nick $chan]))} {
    if {$::pub_setting(noticeacc)} { putserv "NOTICE $nick :This command requires [pub_access_string "o|o"] access to $chan." }
    set check 0
  }
  return $check
}

proc pub_getaccess {user chan} {
  foreach {a b} [list usesubop [expr {([lsearch -exact [channel info $chan] +subop] != -1) ? 1 : 0}] host [getchanhost $user] ouser $user user [nick2hand $user]] { set $a $b }
  if {[isbotnick $user] || [matchattr $user b]} { return "Access for $user ($host) is [pub_bold Bot] ([chattr $user $chan])." }
  if {[matchattr $user n]} { return "Access for $user ($host) is [pub_bold {Global Owner}] ([chattr $user $chan])." }
  if {[matchattr $user |n $chan]} { return "Access for $user ($host) is [pub_bold {Channel Owner}] ([chattr $user $chan])." }
  if {[matchattr $user m]} { return "Access for $user ($host) is [pub_bold {Global Master}] ([chattr $user $chan])." }
  if {[matchattr $user |m $chan]} { return "Access for $user ($host) is [pub_bold {Channel Master}] ([chattr $user $chan])." }
  if {[matchattr $user oa]} { return "Access for $user ($host) is [pub_bold {Global Op}] ([chattr $user $chan])." }
  if {[matchattr $user |oa $chan]} { return "Access for $user ($host) is [pub_bold {Channel Op}] ([chattr $user $chan])." }
  if {$::numversion >= 1061200} {
    if {[matchattr $user ly]} { return "Access for $user ($host) is [pub_bold {Global Halfop}] ([chattr $user $chan])." }
    if {[matchattr $user |ly $chan]} { return "Access for $user ($host) is [pub_bold {Channel Halfop}] ([chattr $user $chan])." }
  }
  if {[pub_issubop $usesubop $user]} { return "Access for $user ($host) is [pub_bold {Global SubOp}] ([chattr $user $chan])." }
  if {[pub_issubop $usesubop $user $chan]} { return "Access for $user ($host) is [pub_bold {Channel SubOp}] ([chattr $user $chan])." }
  if {[matchattr $user vg]} { return "Access for $user ($host) is [pub_bold {Global Voice}] ([chattr $user $chan])." }
  if {[matchattr $user |vg $chan]} { return "Access for $user ($host) is [pub_bold {Channel Voice}] ([chattr $user $chan])." }
  if {[matchattr $user p]} { return "Access for $user ($host) is [pub_bold {Global Partyline}] ([chattr $user $chan])." }
  return "Access for [expr {([pub_cc $user *]) ? "$ouser" : "$user"}] ($host) is [pub_bold None]."
}

proc pub_verifymodes {modestring} {
  if {![regexp {^([\\+|\\-])} [set mstr $modestring] match sign]} { return "Error: Modes must be prefixed with a + or -." }
  regsub -all -- {n|t|i|s|p|m|-|\+} [join [lrange [split $modestring] 0 0]] {} modestring
  if {[llength [lrange [split $mstr] 1 end]] > [string length $modestring]} { return "Error: Too many arguments." }
  if {[llength [lrange [split $mstr] 1 end]] < [regsub -all -- {o|b} $modestring {} modestring]} { return "Error: Too few arguments." }
  return 0
}

proc pub_issubop {usesubop hand {chan ""}} {
  if {[pub_cc $chan ""] || ![validchan $chan]} { return [expr $usesubop && [matchattr $hand $::pub_setting(subop_flag)]] }
  return [expr $usesubop && [matchattr $hand $::pub_setting(subop_flag)|$::pub_setting(subop_flag) $chan]]
}

proc pub_auth {nick uhost hand text} {
  if {![validuser $hand]} {
    putserv "NOTICE $nick :You are not a bot user."
    return
  }
  if {[pub_cc [set pass [join [lrange [split $text] 0 0]]] ""]} {
    putserv "NOTICE $nick :Usage: /msg $::botnick auth <password>"
    return
  }
  if {![passwdok $hand $pass]} {
    putserv "NOTICE $nick :Invalid password."
    return
  }
  if {[matchattr $hand +Q]} {
    putserv "NOTICE $nick :You are already authorized."
    return
  }
  chattr $hand +Q
  putserv "NOTICE $nick :You are now authorized."
}

proc pub_deauth {nick uhost hand text} {
  if {![validuser $hand]} {
    putserv "NOTICE $nick :You are not a bot user."
    return
  }
  if {![matchattr $hand +Q]} {
    putserv "NOTICE $nick :You are not authorized."
    return
  }
  chattr $hand -Q
  putserv "NOTICE $nick :You are no longer authorized."
}

proc pub_deauth_pq {nick uhost hand {chan ""} {text ""}} {
  utimer 2 [list pub_proccess_deauth $nick $hand]
}

proc pub_proccess_deauth {nick hand} {
  if {[validuser $hand] && ![onchan $nick] && [matchattr $hand +Q]} { chattr $hand -Q }
}

proc pub_help_dcc {hand idx text} {
  foreach v [list to so vo ho] { set $v 0 }
  foreach c [channels] {
    if {$::pub_setting(subop)} { set subopchan [expr {([lsearch -exact [channel info $c] +subop] != -1) ? 1 : 0}] }
    set n [expr {([matchattr $hand n|n $c]) ? 1 : 0}]
    set m [expr {([matchattr $hand m|m $c]) ? 1 : 0}]
    if {([matchattr $hand o|o $c] || [isop [hand2nick $hand] $c]) && ![matchattr $hand d|d $c]} { set to 1 }
    if {([matchattr $hand lo|lo $c] || [ishalfop [hand2nick $hand] $c]) && ![matchattr $hand r|r $c]} { set ho 1 }
    if {([matchattr $hand o|o $c] || [isop [hand2nick $hand] $c] || ($subopchan && [matchattr $hand |$::pub_setting(subop_flag) $c])) && ![matchattr $hand d|d $c]} { set so 1 }
    if {[matchattr $hand ov|ov $c] || [isop [hand2nick $hand] $c] || [isvoice [hand2nick $hand] $c] || ($subopchan && [matchattr $hand |$::pub_setting(subop_flag) $c])} { set vo 1 }
  }
  pub_putdcc $idx "Usage: <prefix> <command>"
  pub_putdcc $idx "Commands:"
  if {[matchattr $hand n]} {
    foreach line [list "+Chan <channel> - Adds a channel to the bot's channel list." "-Chan <channel> - Removes a channel from the bot's channel list." "+Host <nick> <host> - Adds a hostmask to a nick's handle." "-Host <nick> <host> - Removes a hostmask from a nick's handle."] { pub_putdcc $idx $line }
  }
  pub_putdcc $idx "Access \[nick\] - Shows the access for a nick."
  if {$m} {
    foreach line [list "Act <action> - Makes the bot preform an action in the channel." "Akick <add/del/list> \[<host\] \[reason\] - Adds, deletes, and lists akicks in the channel." "AOP <add/del> <nick> - Adds or removes a nick from the Channel Op list."] { pub_putdcc $idx $line }
  }
  if {$so} {
    foreach line [list "Autovoice <add/del> <nick> - Adds or deletes a nick from the channel Voice list." "Ban <nick/hostmask> <reason> - Adds a nick or host to the channel ban list."] { pub_putdcc $idx $line }
  }
  if {[matchattr $hand m]} { pub_putdcc $idx "Chanlist - Shows a list of channels the bot is on." }
  if {[matchattr $hand p] || ([matchattr $hand o] && !${::require-p})} { pub_putdcc $idx "Chatme - Makes the bot start a CTCP CHAT session with you." }
  if {[matchattr $hand n]} { pub_putdcc $idx "Chattr <nick> <flags> \[channel\] - Changes the flags of a user." }
  if {[matchattr $hand m]} { pub_putdcc $idx "Cycle \[channel\] - Makes the bot cycle the channel." }
  if {$::numversion >= 1061200} {
    if {$to} { pub_putdcc $idx "Dehalfop - Makes the bot de-halfop a nick." }
    if {$ho} { pub_putdcc $idx "Dehalfopme - Makes the bot de-halfop you." }
  }
  if {$to} {
    foreach line [list "Deop - Makes the bot de-op a nick." "Deopme - Makes the bot de-op you."] { pub_putdcc $idx $line }
  }
  if {$so} { pub_putdcc $idx "Devoice <nick> - Makes the bot de-voice a nick." }
  if {$vo} { pub_putdcc $idx "Devoiceme - Makes the bot de-voice you." }
  if {[matchattr $hand n]} {
    if {$::pub_setting(enable-die) && [matchattr $hand n]} { pub_putdcc $idx "Die - Kills the bot." }
    foreach line [list "Disable <nick> - Prevents a nick from using any of the commands." "Enable <nick> - Allows a nick to use commands after being disabled."] { pub_putdcc $idx $line }
  }
  pub_putdcc $idx "GetHand <nick> - Shows the handle of a nick."
  if {[matchattr $hand m]} {
    foreach line [list "Global <message> - Sends a message to every channel the bot is on." "Hosts <nick> - Shows all hostmasks for a nick's handle."] { pub_putdcc $idx $line }
  }
  if {$::numversion >= 1061200} {
    if {$to} { pub_putdcc $idx "Halfop - Makes the bot halfop a nick." }
    if {$ho} { pub_putdcc $idx "Halfopme - Makes the bot halfop you." }
  }
  foreach line [list "Help - Shows this help message." "Invite \[nick\] <channel> - Invites you or another nick to a channel."] { pub_putdcc $idx $line }
  if {[matchattr $hand m]} { pub_putdcc $idx "Jump \[server\] - Makes the bot change servers." }
  if {$so} { pub_putdcc $idx "Kick <nick> <reason> - Kicks a nick from the channel." }
  if {$n}  { pub_putdcc $idx "Master <add/del> <nick> - Adds or removes a nick from the Channel Master list." }
  if {$to} { pub_putdcc $idx "Mode <modes> - Sets a mode on the channel." }
  if {[matchattr $hand m]} { pub_putdcc $idx "Msg <nick/channel> - Sends a message to a nick or channel." }
  if {$to} {
    foreach line [list "Op <nick> - Makes the bot op a nick." "Opme - Makes the bot op you."] { pub_putdcc $idx $line }
  }
  if {[matchattr $hand n]} {
    foreach line [list "Owner <add/del> <nick> - Adds or removes a nick from the Channel Owner list." "RandPass <nick> - Sets a random password for a nick." "Raw <commands> - Sends raw commands to the irc server."] { pub_putdcc $idx $line }
  }
  if {[matchattr $hand m]} { pub_putdcc $idx "Rehash - Rehashes the bot." }
  if {[matchattr $hand n]} { pub_putdcc $idx "Restart - Restarts the bot." }
  if {$m} {
    foreach line [list "Save - Saves the bot's userfile." "Say - Makes the bot say something in the channel."] { pub_putdcc $idx $line }
  }
  if {[matchattr $hand n]} { pub_putdcc $idx "ShellUptime - Shows the uptime of the shell the bot is running on." }
  if {$::pub_setting(subop) && $to} { pub_putdcc $idx "SubOp - Adds or removes a nick from the channel SubOp list." }
  pub_putdcc $idx "Time - Shows you the current time."
  if {$so} {
    foreach line [list "Topic <new topic> - Sets a new topic on the channel." "Unban <banmask> - Makes the bot unban a banmask."] { pub_putdcc $idx $line }
  }
  pub_putdcc $idx "Unixtime - Shows you the number of seconds that have elapsed since January 1, 1970."
  if {[matchattr $hand m]} {
    foreach line [list "Uptime - Shows the bot's uptime." "User <add/del> <nick> - Adds or removes a nick from the userlist."] { pub_putdcc $idx $line }
  }
  pub_putdcc $idx "Version - Shows the bot's version information."
  if {$so} { pub_putdcc $idx "Voice <nick> - Makes the bot voice a nick." }
  if {$vo} { pub_putdcc $idx "Voiceme - Makes the bot voice you." }
  if {[matchattr $hand n]} { pub_putdcc $idx "Wipe <nick> - Wipes all access from a nick." }
}

proc pub_parsecmd {nick uhost hand chan text} {
  if {[matchattr $hand D]} { return }
  foreach {a b} [list authed [expr {([matchattr $hand +Q] || ([lsearch -exact [channel info $chan] +pubauth] == -1 && !$::pub_setting(allauth))) ? 1 : 0}] usesubop [expr {([lsearch -exact [channel info $chan] +subop] != -1) ? 1 : 0}] botuptime [duration [expr [unixtime] - $::uptime]]] { set $a $b }
  set prefix $::pub_setting(prefix)
  foreach {a b} [list "%botnick" [string tolower $::botnick] "%botnetnick" [string tolower ${::botnet-nick}]] { regsub -all -- $a $prefix $b prefix }
  if {[lsearch -exact [split $prefix] [set newprefix [join [lrange [split [string tolower $text]] 0 0]]]] == -1} { return }
  foreach {a b} [list extxt [lrange [split $text] 1 end] text [join [lrange [split $text] 1 end]]] { set $a $b }
  foreach {a b} [list prefix $newprefix botversion $::pub_setting(version) bothversion $::pub_setting(hversion)] { set $a $b }
  foreach c [list botversion bothversion] {
    foreach {a b} [list "%version" $::version "%ostype" $::tcl_platform(os) "%osversion" $::tcl_platform(osVersion) "%osbyteorder" $::tcl_platform(byteOrder) "%machine" $::tcl_platform(machine) "%platform" $::tcl_platform(platform) "%uptime" $botuptime] { regsub -all -- $a [set $c] $b $c }
    if {[info tclversion] > 8.2} { regsub -all -- "%user" [set $c] $::tcl_platform(user) $c }
  }
  set cmd [string tolower [pub_sindex $extxt 0]]
  if {!$authed} {
    if {$::pub_setting(noticeacc) && [lsearch -exact $::pub_setting(commandlist) $cmd] != -1} { putserv "NOTICE $nick :You are not authorized. Please type \"/msg $::botnick auth <password>\" to authorize yourself." }
    return
  }
  foreach a [list 1 2 3] { set sindex${a} [pub_sindex $extxt ${a}] }
  switch -- $cmd {
    "access" {
      set user [expr {([pub_cc $sindex1 ""]) ? "$nick" : "$sindex1"}]
      pub_putmsg $chan [expr {([onchan $user $chan]) ? "[pub_getaccess $user $chan]" : "$user is not on $chan."}]
    }
    "gethand" {
      if {[pub_cc $sindex1 ""]} {
        puthelp "NOTICE $nick :Usage: $prefix GetHand <nick>"
        return
      }
      if {![onchan $sindex1 $chan]} {
        pub_putmsg $chan "$sindex1 is not on $chan."
        return
      }
      pub_putmsg $chan [expr {([validuser [nick2hand $sindex1]]) ? "$sindex1's handle is [nick2hand $sindex1]." : "$sindex1 is not a bot user."}]
    }
    "invite" {
      if {[pub_cc $sindex1 ""]} {
        puthelp "NOTICE $nick :Usage: $prefix Invite \[nick\] <channel>"
        return
      }
      if {![pub_cc $sindex2 ""]} {
        if {![validchan $sindex2] || ![botonchan $sindex2]} {
          pub_putmsg $chan "I am not on $sindex2."
          return
        }
        if {![onchan $sindex1 $chan]} {
          pub_putmsg $chan "$sindex1 is not on $chan."
          return
        }
        if {[onchan $sindex1 $sindex2]} {
          pub_putmsg $chan "$sindex1 is already on $sindex2."
          return
        }
        putserv "INVITE $sindex1 $sindex2"
        pub_putmsg $chan "$sindex1 has been invited to $sindex2."
        return
      }
      if {![validchan $sindex1] || ![botonchan $sindex1]} {
        pub_putmsg $chan "I am not on $sindex1."
        return
      }
      if {[matchattr $hand o|o $sindex1] && ![matchattr $hand d|d $sindex1]} {
        putserv "INVITE $nick $sindex1"
        return
      }
      if {$::pub_setting(noticeacc)} { putserv "NOTICE $nick :This command requires Channel Op access to $sindex1" }
    }
    "die" {
      if {!$::pub_setting(enable-die)} { return }
      if {![pub_verify_access $hand $nick "n"]} { return }
      die "[lrange $extxt 1 end] ($hand)"
    }
    "chattr" {
      if {![pub_verify_access $hand $nick "n"]} { return }
      if {[pub_cc $sindex2 ""]} {
        puthelp "NOTICE $nick :Usage: $prefix Chattr <nick> <flags> \[channel\]"
        return
      }
      if {![validuser [nick2hand $sindex1]]} {
        pub_putmsg $chan "$sindex1 is not a bot user."
        return
      }
      if {[pub_cc $sindex3 ""]} {
        pub_putmsg $chan "Global flags for $sindex1 are now [chattr [nick2hand $sindex1] $sindex2]."
        return
      }
      pub_putmsg $chan [expr {([validchan $sindex3]) ? "Channel flags for $sindex1 on $sindex3 are now [join [lrange [split [chattr [nick2hand $sindex1] $sindex2 $sindex3] |] 1 1]]." : "$sindex3 is not a valid channel."}]

    }
    "randpass" {
      if {![pub_verify_access $hand $nick "n"]} { return }
      if {[pub_cc $sindex1 ""]} {
        puthelp "NOTICE $nick :Usage: $prefix RandPass <nick>"
        return
      }
      if {![validuser [nick2hand $sindex1]]} {
        pub_putmsg $chan "$sindex1 is not a bot user."
        return
      }
      if {![passwdok [nick2hand $sindex1] ""]} {
        pub_putmsg $chan "$sindex1 already has a password set."
        return
      }
      setuser $sindex1 PASS [encrypt [rand $::pub_setting(rand)] $::pub_setting(key)]
      pub_putmsg $chan "Password set."
    }
    "chatme" {
      if {![pub_verify_access $hand $nick "p"]} { return }
      *ctcp:CHAT $nick $uhost $hand $::botnick CHAT ""
      putserv "NOTICE $nick :The chat request has been sent."
    }
    "+chan" {
      if {![pub_verify_access $hand $nick "n"]} { return }
      if {[pub_cc $sindex1 ""]} {
        puthelp "NOTICE $nick :Usage: $prefix +Chan <channel>"
        return
      }
      if {![regexp {^[\\+!#&]} $sindex1]} {
        pub_putmsg $chan "$sindex1 is not a valid channel."
        return
      }
      if {[validchan $sindex1]} {
        pub_putmsg $chan "$sindex1 is already in the channel list."
        return
      }
      channel add $sindex1
      pub_putmsg $chan "$sindex1 has been added to the channel list."
    }
    "-chan" {
      if {![pub_verify_access $hand $nick "n"]} { return }
      if {[pub_cc $sindex1 ""]} {
        puthelp "NOTICE $nick :Usage: $prefix -Chan <channel>"
        return
      }
      if {![validchan $sindex1]} {
        pub_putmsg $chan "$sindex1 is not in the channel list."
        return
      }
      channel remove $sindex1
      pub_putmsg $chan "$sindex1 has been removed from the channel list."
    }
    "raw" {
      if {![pub_verify_access $hand $nick "n"]} { return }
      if {[pub_cc $sindex1 ""]} {
        puthelp "NOTICE $nick :Usage: $prefix Raw <commands>"
        return
      }
      putserv "[join [lrange $extxt 1 end]]"
      pub_putmsg $chan "Raw commands have been sent."
    }
    "restart" {
      if {![pub_verify_access $hand $nick "n"]} { return }
      putquick "PRIVMSG $chan :Restarting."
      restart
    }
    "disable" {
      if {![pub_verify_access $hand $nick "n"]} { return }
      if {[pub_cc $sindex1 ""]} {
        puthelp "NOTICE $nick :Usage: $prefix Disable <nick>"
        return
      }
      if {![validuser [nick2hand $sindex1]]} {
        pub_putmsg $chan "No handle found for $sindex1."
        return
      }
      if {[matchattr [nick2hand $sindex1] D]} {
        pub_putmsg $chan "[nick2hand $sindex1] is already disabled."
        return
      }
      chattr [nick2hand $sindex1] +D
      pub_putmsg $chan "[nick2hand $sindex1] has been disabled."
    }
    "enable" {
      if {![pub_verify_access $hand $nick "n"]} { return }
      if {[pub_cc $sindex1 ""]} {
        puthelp "NOTICE $nick :Usage: $prefix Enable <nick>"
        return
      }
      if {![validuser [nick2hand $sindex1]]} {
        pub_putmsg $chan "No handle found for $sindex1."
        return
      }
      if {![matchattr [nick2hand $sindex1] D]} {
        pub_putmsg $chan "[nick2hand $sindex1] is already enabled."
        return
      }
      chattr [nick2hand $sindex1] -D
      pub_putmsg $chan "[nick2hand $sindex1] has been enabled."
    }
    "rehash" {
      if {![pub_verify_access $hand $nick "m"]} { return }
      putquick "PRIVMSG $chan :Rehashing."
      rehash
    }
    "jump" {
      if {![pub_verify_access $hand $nick "m"]} { return }
      if {[pub_cc $sindex1 ""]} {
        jump
      } {
        jump $sindex1
      }
    }
    "hosts" {
      if {![pub_verify_access $hand $nick "m"]} { return }
      if {[pub_cc $sindex1 ""]} {
        puthelp "NOTICE $nick :Usage: $prefix Hosts <nick>"
        return
      }
      if {![onchan $sindex1 $chan]} {
        pub_putmsg $chan "$sindex1 is not on $chan."
        return
      }
      if {![validuser [nick2hand $sindex1]]} {
        pub_putmsg $chan "$sindex1 is not a bot user."
        return
      }
      set h [join [getuser [nick2hand $sindex1] HOSTS]]
      pub_putmsg $chan [expr {([pub_cc $h ""]) ? "$sindex1 has no hosts set." : $h}]
    }
    "uptime" {
      if {![pub_verify_access $hand $nick "m|m" $chan]} { return }
      pub_putmsg $chan "Uptime: $botuptime"
    }
    "shelluptime" {
      if {![pub_verify_access $hand $nick "n"]} { return }
      set error [catch {exec uptime} shelluptime]
      pub_putmsg $chan "Uptime: [expr {($error) ? "error" : "$shelluptime"}]"
    }
    "unixtime" { pub_putmsg $chan "[unixtime] seconds have elapsed since January 1, 1970 00:00 GMT." }
    "time" { pub_putmsg $chan "The current time is [clock format [clock seconds] -format "%I:%M %p %D (%Z)"]." }
    "chanlist" {
      if {![pub_verify_access $hand $nick "m"]} { return }
      pub_putmsg $chan "Channel list: [channels]"
    }
    "version" { pub_putmsg $chan "Version: [expr {([matchattr $hand m]) ? "$botversion" : "$bothversion"}]" }
    "+host" {
      if {![pub_verify_access $hand $nick "m"]} { return }
      if {[pub_cc $sindex2 ""]} {
        puthelp "NOTICE $nick :Usage: $prefix +host <nick> <host>"
        return
      }
      if {![validuser $sindex1]} {
        pub_putmsg $chan "$sindex1 is not a bot user."
        return
      }
      setuser $sindex1 HOSTS $sindex2
      pub_putmsg $chan "$sindex2 has been added to $sindex1's host list."
    }
    "-host" {
      if {![pub_verify_access $hand $nick "m"]} { return }
      if {[pub_cc $sindex2 ""]} {
        puthelp "NOTICE $nick :Usage: $prefix -host <nick> <host>"
        return
      }
      if {![validuser $sindex1]} {
        pub_putmsg $chan "$sindex1 is not a bot user."
        return
      }
      if {[set hd [expr {([lsearch -exact [getuser [nick2hand $sindex1] HOSTS] $sindex2] != -1) ? 1 : 0}]]} {
        delhost [nick2hand $sindex1] $sindex2
        return
      }
      pub_putmsg $chan "$sindex2 does not exist on $sindex1's host list."
    }
    "global" {
      if {![pub_verify_access $hand $nick "m"]} { return }
      if {[pub_cc $sindex1 ""]} {
        puthelp "NOTICE $nick :Usage: $prefix Global <message>"
        return
      }
      foreach c [channels] {
        if {[botonchan $c]} { pub_putmsg $c "Global: [join [lrange $extxt 1 end]] ($nick)" }
      }
    }
    "cycle" {
      if {![pub_cc [set chan1 $sindex1] ""]} {
        if {![validchan $chan]} {
          pub_putmsg $chan "$chan1 is not a valid channel."
          return
        }
        set chan $chan1
      }
      if {![pub_verify_access $hand $nick "m|m" $chan]} {
        return
      }
      channel set $chan +inactive
      channel set $chan -inactive
    }
    "user" {
      if {![pub_verify_access $hand $nick "m"]} { return }
      if {[pub_cc [set user $sindex2] ""]} {
        puthelp "NOTICE $nick :Usage: $prefix user <add/del> <nick>"
        return
      }
      if {[pub_cc $sindex1 "add"]} {
        if {![onchan $user $chan]} {
          pub_putmsg $chan "$user is not on $chan."
          return
        }
        if {[validuser [nick2hand $user]]} {
          pub_putmsg $chan "$user is already a bot user."
          return
        }
        set h2add [string range $user 0 [expr $::handlen - 1]]
        adduser $h2add [maskhost [getchanhost $user]]
        pub_putmsg $chan "$user has been added to the userlist."
        putserv "NOTICE $user :You have been added to the userlist. Please set a password by typing \"/msg $::botnick PASS <new password>\"."
        return
      }
      if {[pub_cc $sindex1 "del"]} {
        if {![onchan $user $chan]} {
          pub_putmsg $chan "$user is not on $chan."
          return
        }
        if {![validuser [nick2hand $user]]} {
          pub_putmsg $chan "$user is not a bot user."
          return
        }
        deluser [nick2hand $user]
        return
      }
      puthelp "NOTICE $nick :Usage: $prefix user <add/del> <nick>"
    }
    "subop" {
      if {![pub_verify_access $hand $nick "o|o" $chan]} { return }
      if {!$usesubop} {
        pub_putmsg $chan "SubOp is not enabled for $chan."
        return
      }
      if {[pub_cc [set user $sindex2] ""]} {
        puthelp "NOTICE $nick :Usage: $prefix SubOp <add/del> <nick>"
        return
      }
      if {[pub_cc $sindex1 "add"]} {
        if {![onchan $user $chan]} {
          pub_putmsg $chan "$user is not on $chan."
          return
        }
        if {![validuser [nick2hand $user]]} {
          set h2add [string range $user 0 [expr $::handlen - 1]]
          adduser $h2add [maskhost [getchanhost $user]]
          chattr $h2add |+$::pub_setting(subop_flag) $chan
          if {![isvoice $user $chan]} { pushmode $chan +v $user }
          pub_putmsg $chan "Access for $user has been changed to [pub_bold {Channel SubOp}]."
          putserv "NOTICE $user :Your access for $chan has been changed to [pub_bold {Channel SubOp}]. Please set a password by typing \"/msg $::botnick PASS <new password>\"."
          return
        }
        if {[matchattr [nick2hand $user] |$::pub_setting(subop_flag) $chan]} {
          pub_putmsg $chan "$user already exists on the Channel SubOp list for $chan."
          return
        }
        chattr [nick2hand $user] |+$::pub_setting(subop_flag) $chan
        if {![isvoice $user $chan]} {
          pushmode $chan +v $user
        }
        pub_putmsg $chan "Access for $user has been changed to [pub_bold {Channel SubOp}]."
        if {[passwdok [nick2hand $user] ""]} { putserv "NOTICE $user :Your access for $chan has been changed to [pub_bold {Channel SubOp}]. Please set a password by typing \"/msg $::botnick PASS <new password>\"." }
        return
      }
      if {[pub_cc $sindex1 "del"]} {
        if {![onchan $user $chan]} {
          pub_putmsg $chan "$user is not on $chan."
          return
        }
        if {![validuser [nick2hand $user]] || ![matchattr [nick2hand $user] |$::pub_setting(subop_flag) $chan]} {
          pub_putmsg $chan "$user does not have Channel SubOp access to $chan."
          return
        }
        chattr [nick2hand $user] |-$::pub_setting(subop_flag) $chan
        pub_putmsg $chan "$user has been removed from the Channel SubOp list of $chan."
        return
      }
      puthelp "NOTICE $nick :Usage: $prefix SubOp <add/del> <nick>"
    }
    "aop" {
      if {![pub_verify_access $hand $nick "m|m" $chan]} { return }
      if {[pub_cc [set user $sindex2] ""]} {
        puthelp "NOTICE $nick :Usage: $prefix AOP <add/del> <nick>"
        return
      }
      if {[pub_cc $sindex1 "add"]} {
        if {![onchan $user $chan]} {
          pub_putmsg $chan "$user is not on $chan."
          return
        }
        if {![validuser [nick2hand $user]]} {
          set h2add [string range $user 0 [expr $::handlen - 1]]
          adduser $h2add [maskhost [getchanhost $user]]
          chattr $h2add |+o $chan
          if {![isop $user $chan]} { pushmode $chan +o $user }
          pub_putmsg $chan "Access for $user has been changed to [pub_bold {Channel Op}]."
          putserv "NOTICE $user :Your access for $chan has been changed to [pub_bold {Channel Op}]. Please set a password by typing \"/msg $::botnick PASS <new password>\"."
          return
        }
        if {[matchattr [nick2hand $user] |+o $chan]} {
          pub_putmsg $chan "$user already exists on the Channel Op list for $chan."
          return
        }
        chattr [nick2hand $user] |+o $chan
        if {![isop $user $chan]} { pushmode $chan +o $user }
        pub_putmsg $chan "Access for $user has been changed to [pub_bold {Channel Op}]."
        if {[passwdok [nick2hand $user] ""]} { putserv "NOTICE $user :Your access for $chan has been changed to [pub_bold {Channel Op}]. Please set a password by typing \"/msg $::botnick PASS <new password>\"." }
        return
      }
      if {[pub_cc $sindex1 "del"]} {
        if {![onchan $user $chan]} {
          pub_putmsg $chan "$user is not on $chan."
          return
        }
        if {![validuser [nick2hand $user]] || ![matchattr [nick2hand $user] |o $chan]} {
          pub_putmsg $chan "$user does not have Channel Op access to $chan."
          return
        }
        chattr [nick2hand $user] |-o $chan
        if {[isop $nick $chan]} { pushmode $chan -o $user }
        pub_putmsg $chan "$user has been removed from the Channel Op list of $chan."
        return
      }
      puthelp "NOTICE $nick :Usage: $prefix AOP <add/del> <nick>"
    }
    "master" {
      if {![pub_verify_access $hand $nick "n|n" $chan]} { return }
      if {[pub_cc [set user $sindex2] ""]} {
        puthelp "NOTICE $nick :Usage: $prefix Master <add/del> <nick>"
        return
      }
      if {[pub_cc $sindex1 "add"]} {
        if {![onchan $user $chan]} {
          pub_putmsg $chan "$user is not on $chan."
          return
        }
        if {![validuser [nick2hand $user]]} {
          set h2add [string range $user 0 [expr $::handlen - 1]]
          adduser $h2add [maskhost [getchanhost $user]]
          chattr $h2add |+m $chan
          if {![isop $user $chan]} { pushmode $chan +o $user }
          pub_putmsg $chan "Access for $user has been changed to [pub_bold {Channel Master}]."
          putserv "NOTICE $user :Your access for $chan has been changed to [pub_bold {Channel Master}]. Please set a password by typing \"/msg $::botnick PASS <new password>\"."
          return
        }
        if {[matchattr [nick2hand $user] |+m $chan]} {
          pub_putmsg $chan "$user already exists on the Channel Master list for $chan."
          return
        }
        chattr [nick2hand $user] |+m $chan
        if {![isop $user $chan]} { pushmode $chan +o $user }
        pub_putmsg $chan "Access for $user has been changed to [pub_bold {Channel Master}]."
        if {[passwdok [nick2hand $user] ""]} { putserv "NOTICE $user :Your access for $chan has been changed to [pub_bold {Channel Master}]. Please set a password by typing \"/msg $::botnick PASS <new password>\"." }
        return
      }
      if {[pub_cc $sindex1 "del"]} {
        if {![onchan $user $chan]} {
          pub_putmsg $chan "$user is not on $chan."
          return
        }
        if {![validuser [nick2hand $user]] || ![matchattr [nick2hand $user] |m $chan]} {
          pub_putmsg $chan "$user does not have Channel Master access to $chan."
          return
         }
        chattr [nick2hand $user] |-m $chan
        pub_putmsg $chan "$user has been removed from the Channel Master list of $chan."
        return
      }
      puthelp "NOTICE $nick :Usage: $prefix master <add/del> <nick>"
    }
    "owner" {
      if {![pub_verify_access $hand $nick "n"]} { return }
      if {[pub_cc [set user $sindex2] ""]} {
        puthelp "NOTICE $nick :Usage: $prefix Owner <add/del> <nick>"
        return
      }
      if {[pub_cc $sindex1 "add"]} {
        if {![onchan $user $chan]} {
          pub_putmsg $chan "$user is not on $chan."
          return
        }
        if {![validuser [nick2hand $user]]} {
          set h2add [string range $user 0 [expr $::handlen - 1]]
          adduser $h2add [maskhost [getchanhost $user]]
          chattr $h2add |+n $chan
          if {![isop $user $chan]} { pushmode $chan +o $user }
          pub_putmsg $chan "Access for $user has been changed to [pub_bold {Channel Owner}]."
          putserv "NOTICE $user :Your access for $chan has been changed to [pub_bold {Channel Owner}]. Please set a password by typing \"/msg $::botnick PASS <new password>\"."
          return
        }
        if {[matchattr [nick2hand $user] |+n $chan]} {
          pub_putmsg $chan "$user already exists on the Channel Owner list for $chan."
          return
        }
        chattr [nick2hand $user] |+n $chan
        if {![isop $user $chan]} { pushmode $chan +o $user }
        pub_putmsg $chan "Access for $user has been changed to [pub_bold {Channel Owner}]."
        if {[passwdok [nick2hand $user] ""]} { putserv "NOTICE $user :Your access for $chan has been changed to [pub_bold {Channel Owner}]. Please set a password by typing \"/msg $::botnick PASS <new password>\"." }
        return
      }
      if {[pub_cc $sindex1 "del"]} {
        if {![onchan $user $chan]} {
          pub_putmsg $chan "$user is not on $chan."
          return
        }
        if {![validuser [nick2hand $user]] || ![matchattr [nick2hand $user] |n $chan]} {
          pub_putmsg $chan "$user does not have Channel Owner access to $chan."
          return
        }
        chattr [nick2hand $user] |-n $chan
        pub_putmsg $chan "$user has been removed from the Channel Owner list of $chan."
        return
      }
      puthelp "NOTICE $nick :Usage: $prefix owner <add/del> <nick>"
    }
    "autovoice" {
      if {![pub_verify_access $hand $nick "o|o" $chan]} { return }
      if {[pub_cc [set user $sindex2] ""]} {
        puthelp "NOTICE $nick :Usage: $prefix Autovoice <add/del> <nick>"
        return
      }
      if {[pub_cc $sindex1 "add"]} {
        if {![onchan $user $chan]} {
          pub_putmsg $chan "$user is not on $chan."
          return
        }
        if {![validuser [nick2hand $user]]} {
          adduser [set h2add [string range $user 0 [expr $::handlen - 1]]] [maskhost [getchanhost $user]]
          chattr $h2add |+v $chan
          if {![isvoice $user $chan]} { pushmode $chan +v $user }
          pub_putmsg $chan "Access for $user has been changed to [pub_bold {Channel Voice}]."
          putserv "NOTICE $user :Your access for $chan has been changed to [pub_bold {Channel Voice}]. Please set a password by typing \"/msg $::botnick PASS <new password>\"."
          return
        }
        if {[matchattr [nick2hand $user] |+v $chan]} {
          pub_putmsg $chan "$user already exists on the Channel Voice list for $chan."
          return
        }
        chattr [nick2hand $user] |+v $chan
        if {![isvoice $user $chan]} { pushmode $chan +v $user }
        pub_putmsg $chan "Access for $user has been changed to [pub_bold {Channel Voice}]."
        if {[passwdok [nick2hand $user] ""]} { putserv "NOTICE $user :Your access for $chan has been changed to [pub_bold {Channel Voice}]. Please set a password by typing \"/msg $::botnick PASS <new password>\"." }
        return
      }
      if {[pub_cc $sindex1 "del"]} {
        if {![onchan $user $chan]} {
          pub_putmsg $chan "$user is not on $chan."
          return
        }
        if {![validuser [nick2hand $user]] || ![matchattr [nick2hand $user] |n $chan]} {
          pub_putmsg $chan "$user does not have Channel Voice access to $chan."
          return
        }
        chattr [nick2hand $user] |-v $chan
        if {[isvoice $user $chan]} { pushmode $chan -v $user }
        pub_putmsg $chan "$user has been removed from the Channel Voice list of $chan."
        return
      }
      puthelp "NOTICE $nick :Usage: $prefix autovoice <add/del> <nick>"
    }
    "wipe" {
      if {![pub_verify_access $hand $nick "n"]} { return }
      if {[pub_cc [set mode $sindex2] ""]} {
        puthelp "NOTICE $nick :Usage: $prefix Wipe <nick> <#channel|all|global>"
        return
      }
      set user $sindex1
      if {[matchattr [nick2hand $user] +n]} {
        pub_putmsg $chan "$user has global owner access and cannot be wiped."
        return
      }
      if {![onchan $user $chan]} {
        pub_putmsg $chan "$user is not on $chan."
        return
      }
      if {![validuser [nick2hand $user]]} {
        pub_putmsg $chan "$user has no access."
        return
      }
      switch -- $mode {
        "all" {
          chattr [nick2hand $user] -abcdefghijklmnopqrstuvwxyz
          foreach channel [channels] { chattr [nick2hand $user] |-abcdefghijklmnopqrstuvwxyz $channel }
        }
        "global" { chattr [nick2hand $user] -abcdefghijklmnopqrstuvwxyz }
        default { chattr [nick2hand $user] |-abcdefghijklmnopqrstuvwxyz $mode }
      }
      foreach channel [channels] {
        if {![botonchan $channel] || ![botisop $channel]} { continue }
        if {[isop $user $channel]} { pushmode $channel -o $user }
        if {[ishalfop $user $channel]} { pushmode $channel -h $user }
        if {[isvoice $user $channel]} { pushmode $channel -v $user }
      }
      pub_putmsg $chan "Access for $user has been wiped."
    }
    "act" {
      if {![pub_verify_access $hand $nick "m|m" $chan]} { return }
      if {[pub_cc $sindex1 ""]} {
        puthelp "NOTICE $nick :Usage: $prefix Act <action>"
        return
      }
      putserv "PRIVMSG $chan :\001ACTION [join [lrange $extxt 1 end]]\001"
    }
    "say" {
      if {![pub_verify_access $hand $nick "m|m" $chan]} { return }
      if {[pub_cc $sindex1 ""]} {
        puthelp "NOTICE $nick :Usage: $prefix Say <message>"
        return
      }
      putserv "PRIVMSG $chan :[join [lrange $extxt 1 end]]"
    }
    "akick" {
      if {![pub_verify_access $hand $nick "m|m" $chan]} { return }
      if {[pub_cc $sindex1 ""] || (![pub_cm "*!*@*" $sindex2] && ![pub_cc $sindex1 "list"] && [pub_cc $sindex2 ""])} {
        puthelp "NOTICE $nick :Usage: $prefix Akick <add/del/list> \[<host>\] \[reason\]."
        return
      }
      if {[pub_cc $sindex1 "add"]} {
        if {[isban $sindex2 $chan]} {
          putserv "PRIVMSG $chan :$sindex2 already exists on the Akick list of $chan."
          return
        }
        newchanban $chan $sindex2 $nick [join [lrange $extxt 3 end]] 0
        foreach u [chanlist $chan] {
          if {[pub_ncm $sindex2 $u![getchanhost $u]]} { putserv "KICK $chan $u :[join [lrange $extxt 3 end]]" }
        }
        putserv "PRIVMSG $chan :$sindex2 has been added to the Akick list of $chan."
        return
      }
      if {[pub_cc $sindex1 "del"]} {
        if {![isban $sindex2 $chan]} {
          putserv "PRIVMSG $chan :$sindex2 does not exist on the Akick list of $chan."
          return
        }
        killchanban $chan $sindex2
        putserv "PRIVMSG $chan :$sindex2 has been removed from the Akick list of $chan."
        return
      }
      if {[pub_cc $sindex1 "list"]} {
        if {[pub_cc [join [banlist $chan]] ""]} {
          puthelp "NOTICE $nick :The akick list for $chan is empty."
          return
        }
        puthelp "NOTICE $nick :$chan Akick list:"
        foreach b [banlist $chan] { puthelp "NOTICE $nick :[lindex $b 0] ([lindex $b 5] ([lindex $b 1] ([clock format [lindex $b 3]])))" }
        puthelp "NOTICE $nick :End of Akick list."
      }
    }
    "msg" {
      if {![pub_verify_access $hand $nick "m"]} { return }
      if {[pub_cc $sindex2 ""]} {
        putserv "NOTICE $nick :Usage: $prefix Msg <nick/channel> <message>"
        return
      }
      putserv "PRIVMSG $sindex1 :[join [lrange $extxt 2 end]]"
    }
    "save" {
      if {![pub_verify_access $hand $nick "m|m" $chan]} { return }
      pub_putmsg $chan "Saving channel and user files."
      save
    }
    "mode" {
      if {![pub_verify_access $hand $nick "ood" $chan]} { return }
      if {[pub_cc $sindex1 ""]} {
        putserv "NOTICE $nick :Usage: $prefix Mode <modes>"
        return
      }
      if {[set errorm [pub_verifymodes [set modes [lrange $extxt 1 end]]]] != 0} {
        putserv "NOTICE $nick :$errorm"
        return
      }
      putserv "MODE $chan [join $modes]"
    }
    "voice" {
      if {![pub_verify_access $hand $nick "ood" $chan]} { return }
      if {(![matchattr $hand o|o $chan] && ![isop $nick $chan] && ![pub_issubop $usesubop $hand $chan]) || [matchattr $hand d|d $chan]} {
        if {$::pub_setting(noticeacc)} { putserv "NOTICE $nick :This command requires Channel [expr {($usesubop) ? "SubOp" : "Op"}] access." }
        return
      }
      if {[pub_cc $sindex1 ""]} {
        puthelp "NOTICE $nick :Usage: $prefix Voice <nick>"
        return
      }
      if {![onchan $sindex1 $chan]} {
        pub_putmsg $chan "$sindex1 is not on $chan."
        return
      }
      pushmode $chan +v $sindex1
    }
    "devoice" {
      if {(![matchattr $hand o|o $chan] && ![isop $nick $chan] && ![pub_issubop $usesubop $hand $chan]) || [matchattr $hand d|d $chan]} {
        if {$::pub_setting(noticeacc)} { putserv "NOTICE $nick :This command requires Channel [expr {($usesubop) ? "SubOp" : "Op"}] access." }
        return
      }
      if {[pub_cc $sindex1 ""]} {
        putserv "NOTICE $nick :Usage: $prefix Devoice <nick>"
        return
      }
      if {![onchan $sindex1 $chan]} {
        pub_putmsg $chan "$sindex1 is not on $chan."
        return
      }
      pushmode $chan -v $sindex1
    }
    "op" {
      if {![pub_verify_access $hand $nick "ood" $chan]} { return }
      if {[pub_cc $sindex1 ""]} {
        puthelp "NOTICE $nick :Usage: $prefix Op <nick>"
        return
      }
      if {![onchan $sindex1 $chan]} {
        pub_putmsg $chan "$sindex1 is not on $chan."
        return
      }
      pushmode $chan +o $sindex1
    }
    "halfop" {
      if {$::numversion < 1061200} { return }
      if {![pub_verify_access $hand $nick "ood" $chan]} { return }
      if {[pub_cc [set user $sindex1] ""]} {
        puthelp "NOTICE $nick :Usage: $prefix Halfop <nick>"
        return
      }
      if {![onchan $user $chan]} {
        pub_putmsg $chan "$user is not on $chan."
        return
      }
      pushmode $chan +h $user
    }
    "deop" {
      if {![pub_verify_access $hand $nick "ood" $chan]} { return }
      if {[pub_ncc $sindex1 $::botnick]} {
        pub_putmsg $chan "$::botnick cannot be deopd."
        return
      }
      if {[pub_cc $sindex1 ""]} {
        puthelp "NOTICE $nick :Usage: $prefix Deop <nick>"
        return
      }
      if {![onchan $sindex1 $chan]} {
        pub_putmsg $chan "$sindex1 is not on $chan."
        return
      }
      pushmode $chan -o $sindex1
    }
    "dehalfop" {
      if {$::numversion < 1061200} { return }
      if {![pub_verify_access $hand $nick "ood" $chan]} { return }
      if {[pub_ncm [set user $sindex1] $::botnick]} {
        pub_putmsg $chan "$::botnick cannot be dehalfoped."
        return
      }
      if {[pub_cc $user ""]} {
        puthelp "NOTICE $nick :Usage: $prefix Dehalfop <nick>"
        return
      }
      if {![onchan $user $chan]} {
        pub_putmsg $chan "$user is not on $chan."
        return
      }
      pushmode $chan -h $user
    }
    "deopme" {
      if {![pub_verify_access $hand $nick "ood" $chan]} { return }
      pushmode $chan -o $nick
    }
    "opme" {
      if {![pub_verify_access $hand $nick "ood" $chan]} { return }
      pushmode $chan +o $nick
    }
    "dehalfopme" {
      if {$::numversion < 1061200} { return }
      if {(![matchattr $hand ol|ol $chan] && ![isop $nick $chan] && ![ishalfop $nick $chan]) || [matchattr $hand dr|dr $chan]} {
        if {$::pub_setting(noticeacc)} { putserv "NOTICE $nick :This command requires Channel Halfop access." }
        return
      }
      pushmode $chan -h $nick
    }
    "halfopme" {
      if {$::numversion < 1061200} { return }
      if {(![matchattr $hand ol|ol $chan] && ![isop $nick $chan] && ![ishalfop $nick $chan]) || [matchattr $hand dr|dr $chan]} {
        if {$::pub_setting(noticeacc)} { putserv "NOTICE $nick :This command requires Channel Halfop access." }
        return
      }
      pushmode $chan +h $nick
    }
    "kick" {
      if {(![matchattr $hand o|o $chan] && ![isop $nick $chan] && ![pub_issubop $usesubop $hand $chan]) || [matchattr $hand d|d $chan]} {
        if {$::pub_setting(noticeacc)} { putserv "NOTICE $nick :This command requires Channel [expr {($usesubop) ? "SubOp" : "Op"}] access." }
        return
      }
      if {[pub_cc [set reason [join [lrange $extxt 2 end]]] ""]} {
        puthelp "NOTICE $nick :Usage: $prefix Kick <nick> <reason>"
        return
      }
      set user $sindex1
      if {![matchattr $hand o|o $chan] && ![isop $nick $chan] && [isop $chan]} {
        pub_putmsg $chan "SubOps cannot kick ops."
        return
      }
      if {[pub_ncc $user $::botnick]} {
        pub_putmsg $chan "$::botnick cannot be kicked."
        return
      }
      if {![onchan $user $chan]} {
        pub_putmsg $chan "$user is not on $chan."
        return
      }
      putserv "KICK $chan $sindex1 :$reason ($nick)"
    }
    "ban" {
      if {(![matchattr $hand o|o $chan] && ![isop $nick $chan] && ![pub_issubop $usesubop $hand $chan]) || [matchattr $hand d|d $chan]} {
        if {$::pub_setting(noticeacc)} { putserv "NOTICE $nick :This command requires Channel [expr {($usesubop) ? "SubOp" : "Op"}] access." }
        return
      }
      if {[pub_cc [set reason [join [lrange $extxt 2 end]]] ""]} {
        puthelp "NOTICE $nick :Usage: $prefix Ban <nick/host> <reason>"
        return
      }
      if {![pub_cm "*!*@*" $sindex1]} {
        set user $sindex1
        if {![matchattr $hand o|o $chan] && ![isop $nick $chan] && [isop $user $chan]} {
          pub_putmsg $chan "SubOps cannot ban ops."
          return
        }
        if {![onchan $user $chan]} {
          pub_putmsg $chan "$user is not on $chan."
          return
        }
        pushmode $chan +b [maskhost $user![getchanhost $user]]
        putserv "KICK $chan $user :$reason ($nick)"
        return
      }
      if {![onchan $user $chan]} {
        pub_putmsg $chan "$user is not on $chan."
        return
      }
      if {![matchattr $hand o|o $chan] && ![isop $nick $chan]} {
        foreach u [chanlist $chan] {
          if {[isop $u $chan] && [pub_ncm $user $u![getchanhost $u]]} {
            pub_putmsg $chan "SubOps cannot ban ops."
            return
          }
        }
      }
      pushmode $chan +b $user
    }
    "unban" {
      if {(![matchattr $hand o|o $chan] && ![isop $nick $chan] && ![pub_issubop $usesubop $hand $chan]) || [matchattr $hand d|d $chan]} {
        if {$::pub_setting(noticeacc)} { putserv "NOTICE $nick :This command requires Channel [expr {($usesubop)? "SubOp" : "Op"}] access." }
        return
      }
      if {[pub_cc [set mask $sindex1] ""]} {
        puthelp "NOTICE $nick :Usage: $prefix Unban <banmask>"
        return
      }
      pushmode $chan -b $mask
    }
    "topic" {
      if {(![matchattr $hand o|o $chan] && ![isop $nick $chan] && ![pub_issubop $usesubop $hand $chan]) || [matchattr $hand d|d $chan]} {
        if {$::pub_setting(noticeacc)} { putserv "NOTICE $nick :This command requires Channel [expr {($usesubop) ? "SubOp" : "Op"}] access." }
        return
      }
      if {[pub_cc [set topic [join [lrange $extxt 1 end]]] ""]} {
        puthelp "NOTICE $nick :Usage: $prefix Topic <new topic>"
        return
      }
      putserv "TOPIC $chan :$topic"
    }
    "voiceme" {
      if {![matchattr $hand vo|vo $chan] && ![isvoice $nick $chan] && ![isop $nick $chan] && ![pub_issubop $usesubop $hand $chan]} {
        if {$::pub_setting(noticeacc)} { putserv "NOTICE $nick :This command requires Channel Voice access." }
        return
      }
      pushmode $chan +v $nick
    }
    "devoiceme" {
      if {![matchattr $hand vo|vo $chan] && ![isvoice $nick $chan] && ![isop $nick $chan] && ![pub_issubop $usesubop $hand $chan]} {
        if {$::pub_setting(noticeacc)} { putserv "NOTICE $nick :This command requires Channel Voice access." }
        return
      }
      pushmode $chan -v $nick
    }
    "help" {
      foreach line [list "Usage: $prefix <command>" "Access: [pub_getaccess $nick $chan]" "Commands:"] { puthelp "NOTICE $nick :$line" }
      if {[matchattr $hand n]} {
        foreach line [list "+Chan <channel> - Adds a channel to the bot's channel list." "-Chan <channel> - Removes a channel from the bot's channel list." "+Host <nick> <host> - Adds a hostmask to a nick's handle." "-Host <nick> <host> - Removes a hostmask from a nick's handle."] { puthelp "NOTICE $nick :$line" }
      }
      puthelp "NOTICE $nick :Access \[nick\] - Shows the access for a nick."
      if {[matchattr $hand m|m $chan]} {
        foreach line [list "Act <action> - Makes the bot preform an action in the channel." "Akick <add/del/list> \[host\] \[reason\] - Adds, deletes, and lists akicks in the channel." "AOP <add/del> <nick> - Adds or removes a nick from the Channel Op list."] { puthelp "NOTICE $nick :$line" }
      }
      if {([matchattr $hand o|o $chan] || [isop $nick $chan] || ![pub_issubop $usesubop $hand $chan]) && ![matchattr $hand d|d $chan]} {
        foreach line [list "Autovoice <add/del> <nick> - Adds or deletes a nick from the channel Voice list." "Ban <nick/hostmask> <reason> - Adds a nick or host to the channel ban list."] { puthelp "NOTICE $nick :$line" }
      }
      if {[matchattr $hand m]} { puthelp "NOTICE $nick :Chanlist - Shows a list of channels the bot is on." }
      if {[matchattr $hand p] || ([matchattr $hand o] && !${::require-p})} { puthelp "NOTICE $nick :Chatme - Makes the bot start a CTCP CHAT session with you." }
      if {[matchattr $hand n]} { puthelp "NOTICE $nick :Chattr <nick> <flags> \[channel\] - Changes the flags of a user." }
      if {[matchattr $hand m]} { puthelp "NOTICE $nick :Cycle \[channel\] - Makes the bot cycle the channel." }
      if {$::numversion >= 1061200} {
        if {([matchattr $hand o|o $chan] || [isop $nick $chan]) && ![matchattr $hand d|d $chan]} { puthelp "NOTICE $nick :Dehalfop - Makes the bot de-halfop a nick." }
        if {([matchattr $hand ol|ol $chan] || [isop $nick $chan] || [ishalfop $nick $chan]) && ![matchattr $hand dr|dr $chan]} { puthelp "NOTICE $nick :Dehalfopme - Makes the bot de-halfop you." }
      }
      if {([matchattr $hand o|o $chan] || [isop $nick $chan]) && ![matchattr $hand d|d $chan]} {
        foreach line [list "Deop - Makes the bot de-op a nick." "Deopme - Makes the bot de-op you."] { puthelp "NOTICE $nick :$line" }
      }
      if {([matchattr $hand o|o $chan] || [isop $nick $chan] || [pub_issubop $usesubop $hand $chan]) && ![matchattr $hand d|d $chan]} { puthelp "NOTICE $nick :Devoice <nick> - Makes the bot de-voice a nick." }
      if {[matchattr $hand ov|ov $chan] || [isop $nick $chan] || [isvoice $nick $chan] || [pub_issubop $usesubop $hand $chan]} { puthelp "NOTICE $nick :Devoiceme - Makes the bot de-voice you." }
      if {[matchattr $hand n]} {
        if {$::pub_setting(enable-die) && [matchattr $hand n]} { puthelp "NOTICE $nick :Die - Kills the bot." }
        foreach line [list "Disable <nick> - Prevents a nick from using any of the commands." "Enable <nick> - Allows a nick to use commands after being disabled."] { puthelp "NOTICE $nick :$line" }
      }
      puthelp "NOTICE $nick :GetHand <nick> - Shows the handle of a nick."
      if {[matchattr $hand m]} {
        foreach line [list "Global <message> - Sends a message to every channel the bot is on." "Hosts <nick> - Shows all hostmasks for a nick's handle."] { puthelp "NOTICE $nick :$line" }
      }
      if {$::numversion >= 1061200} {
        if {([matchattr $hand o|o $chan] || [isop $nick $chan]) && ![matchattr $hand d|d $chan]} { puthelp "NOTICE $nick :Halfop - Makes the bot halfop a nick." }
        if {([matchattr $hand ol|ol $chan] || [isop $nick $chan] || [ishalfop $nick $chan]) && ![matchattr $hand dr|dr $chan]} { puthelp "NOTICE $nick :Halfopme - Makes the bot halfop you." }
      }
      foreach line [list "Help - Shows this help message." "Invite \[nick\] <channel> - Invites you or another nick to a channel."] { puthelp "NOTICE $nick :$line" }
      if {[matchattr $hand m]} { puthelp "NOTICE $nick :Jump \[server\] - Makes the bot change servers." }
      if {([matchattr $hand o|o $chan] || [isop $nick $chan] || [pub_issubop $usesubop $hand $chan]) && (![matchattr $hand d|d $chan])} { puthelp "NOTICE $nick :Kick <nick> <reason> - Kicks a nick from the channel." }
      if {[matchattr $hand n|n $chan]} { puthelp "NOTICE $nick :Master <add/del> <nick> - Adds or removes a nick from the Channel Master list." }
      if {([matchattr $hand o|o $chan] || [isop $nick $chan]) && ![matchattr $hand d|d $chan]} { puthelp "NOTICE $nick :Mode <modes> - Sets a mode on the channel." }
      if {[matchattr $hand m]} { puthelp "NOTICE $nick :Msg <nick/channel> - Sends a message to a nick or channel." }
      if {([matchattr $hand o|o $chan] || [isop $nick $chan]) && ![matchattr $hand d|d $chan]} {
        foreach line [list "Op <nick> - Makes the bot op a nick." "Opme - Makes the bot op you."] { puthelp "NOTICE $nick :$line" }
      }
      if {[matchattr $hand n]} {
        foreach line [list "Owner <add/del> <nick> - Adds or removes a nick from the Channel Owner list." "RandPass <nick> - Sets a random password for a nick." "Raw <commands> - Sends raw commands to the irc server."] { puthelp "NOTICE $nick :$line" }
      }
      if {[matchattr $hand m]} { puthelp "NOTICE $nick :Rehash - Rehashes the bot." }
      if {[matchattr $hand n]} { puthelp "NOTICE $nick :Restart - Restarts the bot." }
      if {[matchattr $hand m|m $chan]} {
        foreach line [list "Save - Saves the bot's userfile." "Say - Makes the bot say something in the channel."] { puthelp "NOTICE $nick :$line" }
      }
      if {[matchattr $hand n]} { puthelp "NOTICE $nick :ShellUptime - Shows the uptime of the shell the bot is running on." }
      if {$usesubop && ([matchattr $hand o|o $chan] || [isop $nick $chan]) && ![matchattr $hand d|d $chan]} { puthelp "NOTICE $nick :SubOp - Adds or removes a nick from the channel SubOp list." }
      puthelp "NOTICE $nick :Time - Shows you the current time."
      if {([matchattr $hand o|o $chan] || [isop $nick $chan] || [pub_issubop $usesubop $hand $chan]) && ![matchattr $hand d|d $chan]} {
        foreach line [list "Topic <new topic> - Sets a new topic on the channel." "Unban <banmask> - Makes the bot unban a banmask."] { puthelp "NOTICE $nick :$line" }
      }
      puthelp "NOTICE $nick :Unixtime - Shows you the number of seconds that have elapsed since January 1, 1970."
      if {[matchattr $hand m]} {
        foreach line [list "Uptime - Shows the bot's uptime." "User <add/del> <nick> - Adds or removes a nick from the userlist."] { puthelp "NOTICE $nick :$line" }
      }
      puthelp "NOTICE $nick :Version - Shows the bot's version[expr {([matchattr $hand m]) ? " and operating system." : ""}]."
      if {([matchattr $hand o|o $chan] || [isop $nick $chan] || [pub_issubop $usesubop $hand $chan]) && ![matchattr $hand d|d $chan]} { puthelp "NOTICE $nick :Voice <nick> - Makes the bot voice a nick." }
      if {[matchattr $hand ov|ov $chan] || [isop $nick $chan] || [isvoice $nick $chan] || [pub_issubop $usesubop $hand $chan]} { puthelp "NOTICE $nick :Voiceme - Makes the bot voice you." }
      if {[matchattr $hand n]} { puthelp "NOTICE $nick :Wipe <nick> - Wipes all access from a nick." }
    }
  }
}

putlog "\002PUB:\002 Pub.tcl Version 1.8 by Wcc is loaded."
